@@ -1,6 +1,7 @@
 /* Speed measuring program.
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2008, 2009, 2010 Free
+Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -89,10 +90,10 @@ SPEED_EXTRA_PROTOS2
   } while (0)
 
 
-#if BITS_PER_MP_LIMB == 32
+#if GMP_LIMB_BITS == 32
 #define GMP_NUMB_0xAA  (CNST_LIMB(0xAAAAAAAA) & GMP_NUMB_MASK)
 #endif
-#if BITS_PER_MP_LIMB == 64
+#if GMP_LIMB_BITS == 64
 #define GMP_NUMB_0xAA  (CNST_LIMB(0xAAAAAAAAAAAAAAAA) & GMP_NUMB_MASK)
 #endif
 
@@ -152,8 +153,8 @@ const struct routine_t {
   { "mpn_add_n",         speed_mpn_add_n,     FLAG_R_OPTIONAL },
   { "mpn_sub_n",         speed_mpn_sub_n,     FLAG_R_OPTIONAL },
 
-#if HAVE_NATIVE_mpn_addsub_n
-  { "mpn_addsub_n",      speed_mpn_addsub_n,     FLAG_R_OPTIONAL },
+#if HAVE_NATIVE_mpn_add_n_sub_n
+  { "mpn_add_n_sub_n",      speed_mpn_add_n_sub_n,     FLAG_R_OPTIONAL },
 #endif
 
   { "mpn_addmul_1",      speed_mpn_addmul_1,  FLAG_R },
@@ -197,14 +198,19 @@ const struct routine_t {
   { "mpn_divrem_1c",     speed_mpn_divrem_1c, FLAG_R },
   { "mpn_divrem_1cf",    speed_mpn_divrem_1cf,FLAG_R },
 #endif
-  { "mpn_mod_1",         speed_mpn_mod_1,     FLAG_R },
+  { "mpn_mod_1",         speed_mpn_mod_1,     FLAG_R_OPTIONAL },
 #if HAVE_NATIVE_mpn_mod_1c
-  { "mpn_mod_1c",        speed_mpn_mod_1c,    FLAG_R },
+  { "mpn_mod_1c",        speed_mpn_mod_1c,    FLAG_R_OPTIONAL },
 #endif
   { "mpn_preinv_divrem_1",  speed_mpn_preinv_divrem_1,  FLAG_R },
   { "mpn_preinv_divrem_1f", speed_mpn_preinv_divrem_1f, FLAG_R },
   { "mpn_preinv_mod_1",  speed_mpn_preinv_mod_1, FLAG_R },
 
+  { "mpn_mod_1_1",       speed_mpn_mod_1_1,       FLAG_R_OPTIONAL },
+  { "mpn_mod_1s_2",      speed_mpn_mod_1_2,       FLAG_R_OPTIONAL },
+  { "mpn_mod_1s_3",      speed_mpn_mod_1_3,       FLAG_R_OPTIONAL },
+  { "mpn_mod_1s_4",      speed_mpn_mod_1_4,       FLAG_R_OPTIONAL },
+
   { "mpn_divrem_1_div",  speed_mpn_divrem_1_div,  FLAG_R },
   { "mpn_divrem_1_inv",  speed_mpn_divrem_1_inv,  FLAG_R },
   { "mpn_divrem_1f_div", speed_mpn_divrem_1f_div, FLAG_R },
@@ -219,6 +225,8 @@ const struct routine_t {
   { "mpn_divexact_1",    speed_mpn_divexact_1,    FLAG_R },
   { "mpn_divexact_by3",  speed_mpn_divexact_by3          },
 
+  { "mpn_bdiv_q_1",      speed_mpn_bdiv_q_1,      FLAG_R_OPTIONAL },
+  { "mpn_pi1_bdiv_q_1",  speed_mpn_pi1_bdiv_q_1,  FLAG_R_OPTIONAL },
   { "mpn_bdiv_dbm1c",    speed_mpn_bdiv_dbm1c,    FLAG_R_OPTIONAL },
 
 #if HAVE_NATIVE_mpn_modexact_1_odd
@@ -230,17 +238,8 @@ const struct routine_t {
   { "mpn_mod_34lsub1",   speed_mpn_mod_34lsub1 },
 #endif
 
-  { "mpn_dc_tdiv_qr",       speed_mpn_dc_tdiv_qr       },
-  { "mpn_dc_divrem_n",      speed_mpn_dc_divrem_n      },
-  { "mpn_dc_divrem_sb",     speed_mpn_dc_divrem_sb     },
-  { "mpn_dc_divrem_sb_div", speed_mpn_dc_divrem_sb_div },
-  { "mpn_dc_divrem_sb_inv", speed_mpn_dc_divrem_sb_inv },
-
-  { "mpn_sb_divrem_m3",     speed_mpn_sb_divrem_m3     },
-  { "mpn_sb_divrem_m3_div", speed_mpn_sb_divrem_m3_div },
-  { "mpn_sb_divrem_m3_inv", speed_mpn_sb_divrem_m3_inv },
-
   { "mpn_lshift",        speed_mpn_lshift, FLAG_R   },
+  { "mpn_lshiftc",       speed_mpn_lshiftc, FLAG_R   },
   { "mpn_rshift",        speed_mpn_rshift, FLAG_R   },
 
   { "mpn_and_n",         speed_mpn_and_n,  FLAG_R_OPTIONAL },
@@ -251,7 +250,7 @@ const struct routine_t {
   { "mpn_nior_n",        speed_mpn_nior_n, FLAG_R_OPTIONAL },
   { "mpn_xor_n",         speed_mpn_xor_n,  FLAG_R_OPTIONAL },
   { "mpn_xnor_n",        speed_mpn_xnor_n, FLAG_R_OPTIONAL },
-  { "mpn_com_n",         speed_mpn_com_n            },
+  { "mpn_com",           speed_mpn_com              },
 
   { "mpn_popcount",      speed_mpn_popcount         },
   { "mpn_hamdist",       speed_mpn_hamdist          },
@@ -293,25 +292,59 @@ const struct routine_t {
 #endif
 
   { "mpn_mul_n",         speed_mpn_mul_n            },
-  { "mpn_sqr_n",         speed_mpn_sqr_n            },
+  { "mpn_sqr",           speed_mpn_sqr              },
 
-  { "mpn_kara_mul_n",    speed_mpn_kara_mul_n       },
-  { "mpn_kara_sqr_n",    speed_mpn_kara_sqr_n       },
-  { "mpn_toom3_mul_n",   speed_mpn_toom3_mul_n      },
-  { "mpn_toom3_sqr_n",   speed_mpn_toom3_sqr_n      },
+  { "mpn_toom2_sqr",     speed_mpn_toom2_sqr        },
+  { "mpn_toom3_sqr",     speed_mpn_toom3_sqr        },
+  { "mpn_toom4_sqr",     speed_mpn_toom4_sqr        },
+  { "mpn_toom6_sqr",     speed_mpn_toom6_sqr        },
+  { "mpn_toom8_sqr",     speed_mpn_toom8_sqr        },
+  { "mpn_toom22_mul",    speed_mpn_toom22_mul       },
+  { "mpn_toom33_mul",    speed_mpn_toom33_mul       },
+  { "mpn_toom44_mul",    speed_mpn_toom44_mul       },
+  { "mpn_toom6h_mul",    speed_mpn_toom6h_mul       },
+  { "mpn_toom8h_mul",    speed_mpn_toom8h_mul       },
+  { "mpn_toom32_mul",    speed_mpn_toom32_mul       },
+  { "mpn_toom42_mul",    speed_mpn_toom42_mul       },
+  { "mpn_toom43_mul",    speed_mpn_toom43_mul       },
+  { "mpn_toom63_mul",    speed_mpn_toom63_mul       },
+  { "mpn_nussbaumer_mul",    speed_mpn_nussbaumer_mul    },
+  { "mpn_nussbaumer_mul_sqr",speed_mpn_nussbaumer_mul_sqr},
+#if WANT_OLD_FFT_FULL
   { "mpn_mul_fft_full",      speed_mpn_mul_fft_full      },
   { "mpn_mul_fft_full_sqr",  speed_mpn_mul_fft_full_sqr  },
-
+#endif
   { "mpn_mul_fft",       speed_mpn_mul_fft,     FLAG_R_OPTIONAL },
   { "mpn_mul_fft_sqr",   speed_mpn_mul_fft_sqr, FLAG_R_OPTIONAL },
 
-  { "mpn_mullow_n",      speed_mpn_mullow_n         },
-  { "mpn_mullow_basecase", speed_mpn_mullow_basecase},
+  { "mpn_mullo_n",        speed_mpn_mullo_n         },
+  { "mpn_mullo_basecase", speed_mpn_mullo_basecase  },
 
-  { "mpn_get_str",       speed_mpn_get_str,  FLAG_R_OPTIONAL },
+  { "mpn_bc_mulmod_bnm1",      speed_mpn_bc_mulmod_bnm1      },
+  { "mpn_mulmod_bnm1",         speed_mpn_mulmod_bnm1         },
+  { "mpn_mulmod_bnm1_rounded", speed_mpn_mulmod_bnm1_rounded },
+  { "mpn_sqrmod_bnm1",         speed_mpn_sqrmod_bnm1         },
 
-  { "mpn_set_str",         speed_mpn_set_str,     FLAG_R_OPTIONAL },
-  { "mpn_set_str_basecase",speed_mpn_bc_set_str,  FLAG_R_OPTIONAL },
+  { "mpn_invert",              speed_mpn_invert              },
+  { "mpn_invertappr",          speed_mpn_invertappr          },
+  { "mpn_ni_invertappr",       speed_mpn_ni_invertappr       },
+  { "mpn_binvert",             speed_mpn_binvert             },
+
+  { "mpn_sbpi1_div_qr",        speed_mpn_sbpi1_div_qr,    FLAG_R_OPTIONAL},
+  { "mpn_dcpi1_div_qr",        speed_mpn_dcpi1_div_qr,    FLAG_R_OPTIONAL},
+  { "mpn_mu_div_qr",           speed_mpn_mu_div_qr,       FLAG_R_OPTIONAL},
+  { "mpn_mupi_div_qr",         speed_mpn_mupi_div_qr,     FLAG_R_OPTIONAL},
+  { "mpn_sbpi1_divappr_q",     speed_mpn_sbpi1_divappr_q, FLAG_R_OPTIONAL},
+  { "mpn_dcpi1_divappr_q",     speed_mpn_dcpi1_divappr_q, FLAG_R_OPTIONAL},
+
+  { "mpn_sbpi1_bdiv_qr",       speed_mpn_sbpi1_bdiv_qr       },
+  { "mpn_dcpi1_bdiv_qr",       speed_mpn_dcpi1_bdiv_qr       },
+  { "mpn_sbpi1_bdiv_q",        speed_mpn_sbpi1_bdiv_q        },
+  { "mpn_dcpi1_bdiv_q",        speed_mpn_dcpi1_bdiv_q        },
+
+  { "mpn_get_str",          speed_mpn_get_str,     FLAG_R_OPTIONAL },
+  { "mpn_set_str",          speed_mpn_set_str,     FLAG_R_OPTIONAL },
+  { "mpn_set_str_basecase", speed_mpn_bc_set_str,  FLAG_R_OPTIONAL },
 
   { "mpn_sqrtrem",       speed_mpn_sqrtrem          },
   { "mpn_rootrem",       speed_mpn_rootrem, FLAG_R  },
@@ -332,6 +365,8 @@ const struct routine_t {
 
   { "mpz_mod",           speed_mpz_mod              },
   { "mpn_redc_1",        speed_mpn_redc_1           },
+  { "mpn_redc_2",        speed_mpn_redc_2           },
+  { "mpn_redc_n",        speed_mpn_redc_n           },
 
   { "MPN_COPY",          speed_MPN_COPY             },
   { "MPN_COPY_INCR",     speed_MPN_COPY_INCR        },
@@ -349,6 +384,18 @@ const struct routine_t {
 #if HAVE_NATIVE_mpn_sublsh1_n
   { "mpn_sublsh1_n",     speed_mpn_sublsh1_n        },
 #endif
+#if HAVE_NATIVE_mpn_rsblsh1_n
+  { "mpn_rsblsh1_n",     speed_mpn_rsblsh1_n        },
+#endif
+#if HAVE_NATIVE_mpn_addlsh2_n
+  { "mpn_addlsh2_n",     speed_mpn_addlsh2_n        },
+#endif
+#if HAVE_NATIVE_mpn_sublsh2_n
+  { "mpn_sublsh2_n",     speed_mpn_sublsh2_n        },
+#endif
+#if HAVE_NATIVE_mpn_rsblsh2_n
+  { "mpn_rsblsh2_n",     speed_mpn_rsblsh2_n        },
+#endif
 #if HAVE_NATIVE_mpn_rsh1add_n
   { "mpn_rsh1add_n",     speed_mpn_rsh1add_n        },
 #endif
@@ -779,7 +826,7 @@ run_gnuplot (int argc, char *argv[])
 /* Return a limb with n many one bits (starting from the least significant) */
 
 #define LIMB_ONES(n) \
-  ((n) == BITS_PER_MP_LIMB ? MP_LIMB_T_MAX      \
+  ((n) == GMP_LIMB_BITS ? MP_LIMB_T_MAX      \
     : (n) == 0 ? CNST_LIMB(0)                   \
     : (CNST_LIMB(1) << (n)) - 1)
 
@@ -806,7 +853,7 @@ r_string (const char *s)
       {
         if (siz > 1 || siz < -1)
           printf ("Warning, r parameter %s truncated to %d bits\n",
-                  s_orig, BITS_PER_MP_LIMB);
+                  s_orig, GMP_LIMB_BITS);
         return l;
       }
   }
@@ -819,10 +866,10 @@ r_string (const char *s)
   if (strcmp (s, "bits") == 0)
     {
       mp_limb_t  l;
-      if (n > BITS_PER_MP_LIMB)
+      if (n > GMP_LIMB_BITS)
         {
           fprintf (stderr, "%ld bit parameter invalid (max %d bits)\n",
-                   n, BITS_PER_MP_LIMB);
+                   n, GMP_LIMB_BITS);
           exit (1);
         }
       mpn_random (&l, 1);
@@ -830,10 +877,10 @@ r_string (const char *s)
     }
   else  if (strcmp (s, "ones") == 0)
     {
-      if (n > BITS_PER_MP_LIMB)
+      if (n > GMP_LIMB_BITS)
         {
           fprintf (stderr, "%ld bit parameter invalid (max %d bits)\n",
-                   n, BITS_PER_MP_LIMB);
+                   n, GMP_LIMB_BITS);
           exit (1);
         }
       return LIMB_ONES (n);
